package es.verdnatura.domain

import android.app.ActivityManager
import android.content.Context
import android.content.Intent
import com.google.gson.Gson
import com.google.gson.JsonObject
import es.verdnatura.presentation.base.nameofFunction
import es.verdnatura.presentation.view.feature.login.activity.LoginActivity
import org.json.JSONObject
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response
import kotlin.system.exitProcess


//Se crea interceptor para así generalizar las respuestas y llamadas,
//si hay respuesta de fallo 555 se llama onError que lanza un error para que se haga el catch() directamente,
//si no queremos que haga el catch tenemos que reescribir la llamada onError en la clase donde se llama o incluso
// lanzar un throw error de nuevo para que vuelva al catch y lo ejecute.
//-> sería válido en el caso que el catch fuera lo general para todos ( por ejemplo un Toast y después realizáramos
//otras acciones. Se reescribiria el método onError en LoginViewModel como:
// do_actions-> Haz acciones del fallo
// throw e -> De esta manera se consigue llegar al catch. El error se propaga a la llamada superior.

//Falta pasarle al Callback el tipo que queramos y poder lanzar el toast


abstract class SilexCallback<T>(val context: Context) : Callback<T> {

    override fun onResponse(call: Call<T>, response: Response<T>) {
        if (response.isSuccessful) {
            onSuccess(response)
        } else {
            try {
                if (response.code() == 555) {
                    var message = JSONObject(response.message()).getString("Message")
                    onError(Error(message))
                } else if (response.code() == 401) {
                    if (!isActivityRunning(context, LoginActivity::class.java)) {
                        val intent = Intent(context, LoginActivity::class.java)
                        intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP or Intent.FLAG_ACTIVITY_SINGLE_TOP or Intent.FLAG_ACTIVITY_NEW_TASK)
                        context.startActivity(intent)
                        exitProcess(0)
                    } else {
                        onError(Error(response.message().toString()))
                    }
                } else {

                    onError(Error(errorSalixMessage(response)))
                    //  onError(Error(response.message().toString()))
                }
            } catch (t: Throwable) {
                defaultErrorHandler(t)
            }
        }
    }

    override fun onFailure(call: Call<T>, t: Throwable) {
        try {
            onError(t)
        } catch (t: Throwable) {
            defaultErrorHandler(t)
        }
    }

    private fun defaultErrorHandler(t: Throwable) {
        (nameofFunction((this)) + t.message).toast(context)
        //(context as MobileApplication).playSoundIsOK(false)
    }

    open fun onSuccess(response: Response<T>) {
        // (nameofFunction((this)) + context.getString(R.string.operationSuccessful)).toast(context)
        //(context as MobileApplication).playSoundIsOK(true)
    }

    open fun onError(t: Throwable) {
        throw t
    }

    fun isActivityRunning(context: Context, activityClass: Class<*>): Boolean {
        val packageName = context.packageName
        val activityManager = context.getSystemService(Context.ACTIVITY_SERVICE) as ActivityManager
        val runningActivities = activityManager.getRunningTasks(Int.MAX_VALUE)

        for (taskInfo in runningActivities) {
            if (taskInfo.baseActivity?.className == activityClass.name && taskInfo.topActivity?.packageName == packageName) {
                return true
            }
        }

        return false
    }

    private fun errorSalixMessage(response: Response<T>): String {

        val messageResponse = response.errorBody()!!.string()
        return try {

            val jsonObject = Gson().fromJson(messageResponse, JsonObject::class.java)
            jsonObject?.getAsJsonObject("error")?.get("message")?.asString.toString()

        } catch (ex: Exception) {
            response.message()
        }

    }
}