package es.verdnatura.presentation.view.feature.claim.fragment.reubication.fragment

import android.annotation.SuppressLint
import android.content.Context
import android.view.View.GONE
import android.view.inputmethod.EditorInfo
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentReubicationCollectionBinding
import es.verdnatura.domain.notNull
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnPasillerosItemClickListener
import es.verdnatura.presentation.common.OnReubicationClickListener
import es.verdnatura.presentation.view.component.CustomDialogInputTwoValues
import es.verdnatura.presentation.view.component.CustomDialogThreeButtons
import es.verdnatura.presentation.view.feature.claim.fragment.reubication.adapter.ReubicatorAdapter
import es.verdnatura.presentation.view.feature.claim.fragment.reubication.model.Reubication
import es.verdnatura.presentation.view.feature.ubicador.fragment.UbicadorViewModel

@Suppress("UNUSED_ANONYMOUS_PARAMETER")
class ReubicationCollectionFragment(
    var list: MutableList<Reubication>, var shelving: String
) : BaseFragment<FragmentReubicationCollectionBinding, UbicadorViewModel>(
    UbicadorViewModel::class
) {
    private var pasillerosItemClickListener: OnPasillerosItemClickListener? = null
    private var adapter: ReubicatorAdapter? = null
    private lateinit var customDialogThreeButtons: CustomDialogThreeButtons
    private lateinit var customDialogInputTwoValues: CustomDialogInputTwoValues
    private lateinit var listItems: ArrayList<Reubication>

    companion object {
        fun newInstance(
            entryPoint: MutableList<Reubication>, shelving: String
        ): ReubicationCollectionFragment {

            return ReubicationCollectionFragment(entryPoint, shelving)
        }
    }

    override fun getLayoutId(): Int = R.layout.fragment_reubication_collection

    override fun onAttach(context: Context) {
        //sergio: necesario para pasarle el context al recyclerViewer
        if (context is OnPasillerosItemClickListener) pasillerosItemClickListener = context
        super.onAttach(context)
    }

    @SuppressLint("SetTextI18n")
    override fun init() {
        customDialogInputTwoValues = CustomDialogInputTwoValues(requireContext())
        ma.hideBottomNavigation(GONE)

        binding.mainToolbar.toolbarTitle.text =
            "${getString(R.string.wagoon)}:${shelving.uppercase()}"
        setEvents()
        showList(list)
        super.init()
    }

    private fun showList(list: List<Reubication>) {

        for (item in list) {
            item.code = shelving.uppercase()
            if (item.carros.isNotEmpty()) {
                for (i in item.carros) {
                    i.visible = "(" + i.visible + ")"
                }
            }
        }

        adapter = ReubicatorAdapter(list.sortedWith(compareBy { item ->
            item.carros.getOrNull(0)?.pickingOrder ?: 0
        }), pasillerosItemClickListener!!, object : OnReubicationClickListener {
            override fun onReubicationClick(reubication: Reubication) {
                showLeaveItemDialog(reubication)
            }

        })
        binding.reubicatorCollectionsRecycler.adapter = adapter
        binding.reubicatorCollectionsRecycler.layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)

    }

    private fun setEvents() {
        binding.mainToolbar.backButton.setOnClickListener {
            ma.onMyBackPressed()
        }

        binding.collectionSwipe.setOnRefreshListener {
            viewModel.itemShelvingAlternative(
                shelvingFk = shelving,
            )
            binding.collectionSwipe.isRefreshing = false

        }

        binding.scanInput.requestFocus()
        binding.scanInput.setOnEditorActionListener { _, actionId, _ ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5) {

                if (!binding.scanInput.text.isNullOrEmpty()) {
                    if (list.isNotEmpty()) {
                        scanPlate(binding.scanInput.text.toString().uppercase())
                    } else {
                        shelving = binding.scanInput.text.toString().uppercase()
                        viewModel.itemShelvingAlternative(
                            shelvingFk = shelving,
                        )
                    }
                }

                binding.scanInput.setText("")
                ma.hideKeyboard(binding.scanInput)
                return@setOnEditorActionListener true
            }
            false
        }

    }

    private fun scanPlate(plate: String) {
        var found = false
        all_list@ for (items in list) {
            for (placements in items.carros) {
                if (placements.shelvingFk == plate) {
                    found = true
                    showLeaveItemDialog(items)
                    break@all_list
                }

            }
        }
        if (!found) {
            ma.messageWithSound(
                getString(R.string.itemScanetNotFound),
                isError = true,
                isPlayed = true,
                isToasted = true
            )
        }
    }

    private fun showLeaveItemDialog(itemReubication: Reubication) {

        val messageDescrip =
            getString(R.string.scanPlateToLeave) + itemReubication.itemFk + "-" + itemReubication.longName

        customDialogInputTwoValues.setEnableValue(false)
        customDialogInputTwoValues.setInfoTex(getString(R.string.quantity))
        customDialogInputTwoValues.setInfoTextTwo(getString(R.string.shelving))

        customDialogInputTwoValues.setDescription(messageDescrip)
            .setValue(itemReubication.quantity)
            .setOkButton(getString(R.string.leaveItem)) {

                customDialogMerge(itemReubication)

            }.setKoButton(getString(R.string.cancel)) {
                customDialogInputTwoValues.setValueTwo("")
                scanRequest()
                customDialogInputTwoValues.dismiss()
            }.show()

        customDialogInputTwoValues.getEditTextTwo()
            .setOnEditorActionListener { v, actionId, event ->
                if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {

                    customDialogMerge(itemReubication)

                    return@setOnEditorActionListener true
                }
                false
            }

        customDialogInputTwoValues.setFocusTextTwo()

    }

    private fun customDialogMerge(itemReubication: Reubication) {

        if (customDialogInputTwoValues.getValueTwo().isNotEmpty()) {
            viewModel.itemShelvingMerge(
                itemReubication.id, customDialogInputTwoValues.getValueTwo().uppercase()
            )
        } else {
            ma.messageWithSound(
                getString(R.string.returnScan), isError = true, isPlayed = true, isToasted = true
            )
        }
        customDialogInputTwoValues.setValue("")
        customDialogInputTwoValues.setValueTwo("")
        customDialogInputTwoValues.dismiss()
        scanRequest()

    }

    private fun scanRequest() {
        binding.scanInput.requestFocus()

    }

    override fun observeViewModel() {
        with(viewModel) {

            loadReubicationList.observe(viewLifecycleOwner) { event ->
                event.getContentIfNotHandled().notNull {

                    if (it.list.isEmpty()) {

                        ma.messageWithSound(
                            getString(R.string.successReubication),
                            isError = false,
                            isPlayed = true,
                            getString(
                                R.string.info
                            )
                        )
                        ma.onMyBackPressed()
                    } else {
                        showList(it.list)
                    }

                }
            }

            response.observe(viewLifecycleOwner) {
                viewModel.itemShelvingAlternative(
                    shelvingFk = shelving,
                )
            }

        }
    }

}