package es.verdnatura.presentation.view.feature.controlador.fragment

import android.annotation.SuppressLint
import android.content.Context
import android.graphics.drawable.Drawable
import android.os.Bundle
import android.print.PrintAttributes
import android.print.PrintManager
import android.view.GestureDetector
import android.view.MotionEvent
import android.view.View
import android.webkit.WebChromeClient
import android.webkit.WebView
import android.widget.ImageView
import androidx.core.view.GestureDetectorCompat
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentWebBinding
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnOptionsSelectedListener
import es.verdnatura.presentation.common.ToolBarAdapterTooltip
import es.verdnatura.presentation.view.component.CustomDialog
import org.json.JSONObject
import kotlin.math.abs

@Suppress("UNUSED_ANONYMOUS_PARAMETER")
class WebFragment(
    var entryPoint: String = ""
) : BaseFragment<FragmentWebBinding, ControladorViewModel>(ControladorViewModel::class) {
    private lateinit var gestureDetector: GestureDetectorCompat
    override fun getLayoutId(): Int = R.layout.fragment_web
    private lateinit var customDialog: CustomDialog

    companion object {
        fun newInstance(entryPoint: String) = WebFragment(entryPoint)
    }

    override fun init() {

        customDialog = CustomDialog(requireContext())
        setToolbar()
        setWeb()
        super.init()
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {

        gestureDetector = GestureDetectorCompat(
            requireActivity(),
            object : GestureDetector.SimpleOnGestureListener() {
                override fun onFling(
                    e1: MotionEvent?,
                    e2: MotionEvent,
                    velocityX: Float,
                    velocityY: Float
                ): Boolean {
                    val deltaX = (e2.x - e1?.x!!)
                    val deltaY = (e2.y - e1.y)
                    val deltaAbsX = abs(deltaX)
                    val deltaAbsY = abs(deltaY)
                    if (deltaAbsX > deltaAbsY) {
                        if (velocityX > 2000 && velocityY < 500 && velocityY>=0) {

                            binding.webView.goBack()
                            return true

                        }

                    }
                    return false
                }
            }
        )

        super.onViewCreated(view, savedInstanceState)
    }

    @SuppressLint("ClickableViewAccessibility", "SetJavaScriptEnabled")
    private fun setWeb() {

        binding.webView.webChromeClient = WebChromeClient()

        val webSettings = binding.webView.settings
        webSettings.javaScriptEnabled = true
        webSettings.domStorageEnabled = true
        webSettings.loadWithOverviewMode = true
        webSettings.useWideViewPort = true
        webSettings.builtInZoomControls = true
        webSettings.displayZoomControls = false
        webSettings.setSupportZoom(true)
        webSettings.allowFileAccess = true

        binding.webView.setOnTouchListener { v, event ->
            gestureDetector.onTouchEvent(event)
        }

        val dataScanned = JSONObject(entryPoint)

        if (dataScanned.toString().contains("cmr?")) {
            val pdfUrl = "${dataScanned.get("web")}"
            val googleDocsViewerUrl = "https://docs.google.com/gview?embedded=true&url=$pdfUrl"
            binding.webView.loadUrl(googleDocsViewerUrl)
        } else {
            binding.webView.loadUrl(dataScanned.get("web").toString())
        }

    }

    private fun setToolbar() {

        binding.mainToolbar.toolbarTitle.text =
            buildString {
                append(getString(R.string.item))
                append(JSONObject(entryPoint).get("entryPoint").toString())
            }

        val listIcons: ArrayList<ImageView> = ArrayList()
        val iconPrint = ImageView(context)
        iconPrint.setImageResource(R.drawable.ic_print_black_24dp)
        iconPrint.tooltipText = getTooltip(R.drawable.ic_print_black_24dp)

        //listIcons.add(iconPrint)

        binding.mainToolbar.toolbarIcons.adapter =
            ToolBarAdapterTooltip(listIcons, object : OnOptionsSelectedListener {
                override fun onOptionsItemSelected(item: Drawable) {

                    when (item) {
                        iconPrint.drawable -> createWebPrintJob(binding.webView)
                    }

                }
            })
        binding.mainToolbar.toolbarIcons.layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)

    }

    private fun createWebPrintJob(webView: WebView) {

        // Get a PrintManager instance
        (activity?.getSystemService(Context.PRINT_SERVICE) as? PrintManager)?.let { printManager ->

            val jobName = "${getString(R.string.app_name)} Document"

            // Get a print adapter instance
            val printAdapter = webView.createPrintDocumentAdapter(jobName)

            // Create a print job with name and adapter instance
            printManager.print(
                jobName,
                printAdapter,
                PrintAttributes.Builder().build()
            ).also { printJob ->
                println("job printed")
                // Save the job object for later status checking
                // printJobs += printJob
            }
        }
    }

}

